<?php

namespace Intergo\Providers\Enums;

use Intergo\Providers\MessageManager\Drivers\MstatViberDriver;
use Intergo\Providers\MessageManager\Drivers\NthViberDriver;

class FailedReasonEnum extends BaseEnum
{
    const DEFAULT_CUSTOMER_FAILED_REASON = 'Technical error 5008 - Contact Support';

    // ITSMPP ERROR RESPONSES ON THE API EG. resulted in a `400 Bad Request` response:\n{\"error\":\"no_money\"}
    const NO_ACTIVE_AGGREGATOR = "no_active_aggregator";
    const USER_NOT_ALLOWED_SENDING_TO_THIS_AGGREGATOR = "user_not_allowed_sending_to_this_aggregator";
    const NO_MONEY = "no_money";
    const NO_SOURCE_PRICE = "no_source_price";
    const NO_MCC_MNC = "no_mcc_mnc";
    const NO_CLIENT_PRICE = "no_client_price";

    //ALARIS ERROR RESPONSES ON THE API EG. resulted in a `400 Bad Request` response: NO CLT RATE
    const CLT_AGREEMENT_EXPIRED = "CLT AGREEMENT EXPIRED";
    const NO_CLT_RATE = "NO CLT RATE";
    const NO_VDR_RATE = "NO VDR RATE";
    const NO_CLT_CUR = "NO CLT CUR";
    const NO_MCCMNC_FOR_DNIS = "NO MCCMNC FOR DNIS";
    const CLT_INBOUND_TRAFFIC_IS_PRHBT = "CLT INBOUND TRAFFIC IS PRHBT";
    const DISABLED_CHANNEL = "DISABLED CHANNEL";
    const CLT_PRDCT_NOT_FOUND = "CLT PRDCT NOT FOUND";
    const MSG_LMT_EXCEED_CLT_PRDCT_BLK = "MSG LMT EXCEED, CLT PRDCT BLK";
    const ROUTE_FOUND_FILTERED_BY_RULES = "ROUTE FOUND FILTERED BY RULES";
    const CLT_CARRIER_NOT_FOUND = "CLT CARRIER NOT FOUND";
    const CLT_ACC_NOT_FOUND = "CLT ACC NOT FOUND";
    const CLT_CL_EXCEEDED = "CLT CL EXCEEDED";
    const CLT_CHNL_OR_POI_NOT_FOUND = "CLT CHNL OR POI NOT FOUND";
    const CLT_POI_EXPIRED = "CLT POI EXPIRED";
    const ROUTE_FND_FLTRD_BY_VD_CHL_FLTR = "ROUTE FND FLTRD BY VD CHL FLTR";
    const NO_ROUTES_FOUND = "NO ROUTES FOUND";
    const ROUTE_FLTRD_OR_BLCKD = "ROUTE FLTRD OR BLCKD";
    const CLT_LOW_MPS_LIMIT = "CLT LOW MPS LIMIT";
    const CLT_PRDCT_LOCKED = "CLT PRDCT LOCKED";
    const IM_CHNL_INVALID = "IM CHNL INVALID";
    const IM_CHNL_REJECTED = "IM CHNL REJECTED";
    const UNKNOWN_ERROR = "unknown_error";

    public static $failed_code_description = [
        self::CLT_AGREEMENT_EXPIRED =>"Client agreement is outdated",
        self::NO_CLT_RATE =>"Client rate not found",
        self::NO_VDR_RATE =>"No vendor rates found",
        self::NO_CLT_CUR =>"Cannot find client account currency",
        self::NO_MCCMNC_FOR_DNIS =>"Cannot find MCCMNC for DNIS",
        self::CLT_INBOUND_TRAFFIC_IS_PRHBT =>"Client inbound traffic is prohibited",
        self::DISABLED_CHANNEL =>"Client channel disabled",
        self::CLT_PRDCT_NOT_FOUND =>"Client product not found",
        self::MSG_LMT_EXCEED_CLT_PRDCT_BLK =>"Message limit exceeded, client product blocked",
        self::ROUTE_FOUND_FILTERED_BY_RULES =>"Routes found but filtered by rules",
        self::CLT_CARRIER_NOT_FOUND =>"Client carrier not found",
        self::CLT_ACC_NOT_FOUND =>"Client account not found",
        self::CLT_CL_EXCEEDED =>"Client credit limit exceeded",
        self::CLT_CHNL_OR_POI_NOT_FOUND =>"Client POI or channel not found",
        self::CLT_POI_EXPIRED =>"Client POI is outdated",
        self::ROUTE_FND_FLTRD_BY_VD_CHL_FLTR =>"Routes found but filtered by vendor channel filter",
        self::NO_ROUTES_FOUND =>"No routes found",
        self::ROUTE_FLTRD_OR_BLCKD =>"Routes filtered out by blocking rules",
        self::CLT_LOW_MPS_LIMIT =>"Client low balance MPS limit",
        self::CLT_PRDCT_LOCKED =>"Client product is locked",
        self::IM_CHNL_INVALID =>"IM channel invalid",
        self::IM_CHNL_REJECTED => "IM channels rejected"
    ];

    /**
     * Get the error type so we can use it on Graylog or other places to group the rejected errors from providers
     * @param $reason
     * @return mixed|string
     */
    public static function getRejectedErrorType($reason)
    {
        $error_types = self::getConstantValues();
        foreach($error_types as $error_type){
            if(strstr($reason, $error_type) !== FALSE){
                return self::$failed_code_description[$error_type] ?? $error_type;
            }
        }
        return self::UNKNOWN_ERROR;
    }

    /**
     * Get the error type so we can use it on Graylog or other places to group the rejected errors from providers
     * @param $reason
     * @return mixed|string
     */
    public static function getViberFailedReason(string $viber_status_code)
    {
        switch ($viber_status_code) {
            case strstr($viber_status_code, ViberCommonStatusEnum::USER_BLOCKED) !== FALSE :
                return 'Technical error 5009 - Communication is blocked by recipient';
            case strstr($viber_status_code, ViberCommonStatusEnum::NOT_VIBER_USER) !== FALSE :
                return 'Technical error 5010 - User does not have Viber application installed on mobile device';
            case strstr($viber_status_code, ViberCommonStatusEnum::NO_SUITABLE_DEVICE) !== FALSE :
                return 'Technical error 5011 - Message not supported by mobile device';
            case strstr($viber_status_code, NthViberDriver::TIMEOUT) !== FALSE :
                return 'Technical error 5012 - Request timeout';
            case strstr($viber_status_code, NthViberDriver::INVALID_PHONE_NUMBER_FORMAT) !== FALSE :
            case strstr($viber_status_code, MstatViberDriver::DESTINATION_INVALID) !== FALSE :
                return 'Technical error 5013 - Invalid destination address (MSISDN), e.g. MSISDN not according to E.164 standard';
            case strstr($viber_status_code, NthViberDriver::REQUEST_ERROR) !== FALSE :
                return 'Technical error 5014 - Contact Support';
            case strstr($viber_status_code, NthViberDriver::INTERNAL_ERROR) !== FALSE :
                return 'Technical error 5015 - Contact Support';
        }
        return null;
    }

    /**
     * Get the 'failed_reason' that will be displayed to the customer.
     *
     * Here we map the error messages/statuses from providers with the error messages that will be visible to customers.
     *
     * @see https://app.asana.com/0/754800746265705/1201445985948718/f
     * @see https://intergo.atlassian.net/wiki/spaces/SMSD/pages/1660387332/User+Error+Codes
     * @param string $providerReason
     * @return string
     */
    public static function getCustomerFailedReason(string $providerReason)
    {

        $internal_reason = self::getViberFailedReason($providerReason);

        if ($internal_reason ) {
            return  $internal_reason;
        }

        switch ($providerReason) {
            // Routee specific: https://docs.routee.net/docs/statuses-callbacks
            case 'Undelivered on Handset':
                return 'Technical error 5001 - Undelivered on Handset';
            case 'Unknown Number':
                return 'Technical error 5002 - Unknown Number';
            case 'Roaming':
                return 'Technical error 5003 - Roaming';
            case 'Call Barred':
                return 'Technical error 5004 - Call Barred';
            case 'Unknown Status':
                return 'Technical error 5005 - Unknown Status';
            case 'Carrier Violation':
                return 'Technical error 5006 - Carrier Violation';
            case 'Expired':
                return 'Technical error 5007 - Expired';
            // If the error message/status from provider is not recognized/mapped:
            default:
                return self::DEFAULT_CUSTOMER_FAILED_REASON;
        }
    }
}
